#!/bin/sh

#
# Install script for the TSI
#
#   Copies platform specific TSI files into a user-chosen directory
#
# Two groups of TSIs are known:
#   TSI without batch sub-system: composed from tsi/SHARED and
#                                 specific files from tsi/NOBATCH
#   TSI with batch sub-system: composed from tsi/SHARED and
#                              specific files from tsi/<specific_system>
#
#
# Called without parameters, the user will be asked which TSI to install.
# 
# This script can also be called non-interactively using
#   
#   Install.sh <tsi_choice> <install_dir>
# 
# where 'tsi_choice' is one of the platform specific TSIs and 
# 'install_dir' is the destination directory. 
# 
# Example:
#   Install.sh tsi/tsi_NOBATCH ./tsi_NOBATCH
# will install the nobatch TSI into 'tsi_NOBATCH'
#

nobatch_tsi=tsi/NOBATCH
shared_tsi=tsi/SHARED

#
# get the list of all TSIs
#
alltsis=`ls -fd tsi/* tsi_contrib/* `

#
# list all TSIs
#
listTSIs() {
  echo ""
  echo "Available TSI implementations are:"
  echo ""
  i=0
  for tsi in $alltsis; do
    if [ $tsi != $shared_tsi ]; then
      i=`expr $i + 1`
      echo "$i: $tsi"
    fi
  done
}

#
# let the user select one TSI for installation
#
choose() {
  echo ""
  echo "The installation will copy all required files into a new directory."
  echo "Files common to all TSI implementations are copied from tsi/bin,"
  echo "tsi/conf and tsi/SHARED."
  echo "The TSI without a batch sub-system requires also files from tsi/NOBATCH."
  echo "Each TSI with a batch sub-system needs the files from tsi/<specific_system>."
  echo ""
  echo "Select a TSI to install (enter number)"
  read input
  i=0
  for tsi in $alltsis; do
    if [ "$tsi" != "$shared_tsi" ]; then
      i=`expr $i + 1`
      if [ $i = $input ]; then
        break
      fi
    fi
  done
}

#
# get the installation directory from the user
#
getdir() {
  installdir=/usr/share/unicore/$tsi
  echo ""
  echo "Enter installation directory for $tsi (relative or absolute path) [$installdir]"
  read indir
  if [ "$indir" != "" ]; then
    installdir=$indir
  fi
}

#
# let the user confirm the installation
#
confirm_tsi() {
  echo ""
  echo "Please confirm installation of $1 into directory $2 by typing y/n [y]"
  read yesno
  case $yesno in
    y*|Y*) echo "";;
    "") echo "";;
    n*|N*) echo "Installation stopped because of missing confirmation"; exit 1;;
    *) echo "Installation stopped because of doubtful confirmation"; exit 1;;
  esac
}

#
# copy files from directory $1 to $2
#
copy_dir() {
  for file in $1/*; do
    echo "Copy $file to $2/`basename $file`"
    cp $file $2
  done
}

#
# installation


tsi=$1
if [ "$tsi" = "" ]; then
  listTSIs
  choose
fi

installdir=$2
if [ "$installdir" = "" ]; then
  getdir
  confirm_tsi $tsi $installdir
fi


#
# test existence of $installdir
#
if [ -d $installdir ]; then
  if [ $2 == "" ]; then
    echo "Installation directory $installdir already exists"
    confirm_tsi $tsi $installdir
    echo ""
    echo "Files from $installdir will be overwritten"
    echo ""
    chmod -R u+w $installdir
  fi
elif [ -f $installdir ]; then
  echo "Installation directory $installdir identical to existing file."
  echo "Installation stopped because of doubtful confirmation,"
  echo "Remove file $installdir first."
fi

#
# generate installation directory
#
mkdir -p $installdir
mkdir -p $installdir/logs
mkdir -p $installdir/bin
mkdir -p $installdir/conf
mkdir -p $installdir/perl

#
# copy common files
#
echo "Copy common files (bin,conf) first:"
copy_dir bin $installdir/bin
copy_dir conf $installdir/conf

#
# copy shared files
#
echo "Copy shared files (common to all installations):"
copy_dir $shared_tsi $installdir/perl

#
# copy specific files
#
echo ""
echo "Copy (overwrite) specific files:"
copy_dir $tsi $installdir/perl

#
# restrict file permissions to owner
#
chmod 711 $installdir
chmod 600 $installdir/*
chmod 711 $installdir/perl
chmod 644 $installdir/perl/tsi_ls
chmod 644 $installdir/perl/tsi_df
chmod 711 $installdir/logs

# ensure helper scripts are executable
chmod 750 $installdir/bin
chmod 750 $installdir/conf


echo ""
echo "##########################################################"
if [ "$tsi" = "$nobatch_tsi" ]; then
  echo "NOTE: for NON-LINUX systems, copy the correct variation of"
  echo "  $installdir/perl/tsi.*"
  echo "to $installdir/tsi"
fi
echo "Finish installation by editing $installdir/perl/tsi."
echo "and $installdir/conf/tsi.properties."
echo "Execute \"Install_permissions.sh $installdir\" after update."
echo "Finally start the TSI as described in the README file"
echo "##########################################################"
echo ""
echo "NB: When porting to new platform all specific files have to be adapted!"
echo ""
