package EndProcessing;

require Exporter;
@ISA = qw(Exporter);

@EXPORT_OK = qw(end_processing);

use Reporting qw(debug_report start_report ok_report failed_report command_report);

use strict;

# Processing at the (possible) end of a BSS job

# GENERIC

#
# Called in response to a TSI_ENDPROCESSING request from the NJS
#
# Used to detect if a BSS job has finished executing (NJS thinks
# that it has because it has disappeared from qstat but this may
# not be correct - moving between queues and also moves stder/stdout
# to the Outcome directory.
#
# arg1: text sent by the NJS. this is processed for:
#           TSI_BSSID - the identifier of the BSS job which the NJS thinks
#                           has finished
#           TSI_OUTCOME_DIRECTORY - directory for the stdout and stderr files
#                           "NONE" is interpreted as delete the files
#
# returns: void to TSI
#          if job has completed, then returns the line from
#            stderr that contains the exit code of the user's job
#          if job has not completed returns TSI_STILLEXECUTING
#
# side effect: moves stdout and stderr of completed job to TSI_OUTCOME_DIRECTORY
#
sub end_processing {

    my ($junk, $out_name, $err_name);

    my $from_njs = shift;

    my $requestid;
    my $outcome_directory;
    my $disposition = "KEEP";
    
    command_report("Testing for end of job.");
    
    $_ = $from_njs;
    
    while(/#TSI_(\S+) (.*)\n/g) {
	  $requestid         = $2 if $1 eq "BSSID";
	  $outcome_directory = $2 if $1 eq "OUTCOME_DIR";
	  $disposition       = $2 if $1 eq "DISPOSITION";
	}
      
      $outcome_directory =~ s/\$USER/$ENV{USER}/g;
    $outcome_directory =~ s/\$HOME/$ENV{HOME}/g;
    
    start_report("End detecting for: $requestid (stdout and stderr in $outcome_directory)");
    
    # Perl is over paranoid, at this point we are always
    # set to a user, so this is safe. Untaint the data.
    if( $outcome_directory =~ /(.+)/ ) {
        $outcome_directory = $1;
    }

    # The BSS was requested to put stdout and stderr in these files (see Submit.pm)
    $out_name = "$outcome_directory/stdout";
    $err_name = "$outcome_directory/stderr";
    
    # Are the files there? They may not be as the NJS
    # could be wrong in thinking that the job is done.
    #
    # Assume here that the BSS always returns _both_ stdout and stderr
    # files to here, if only one is returned the NJS will never think
    # that the job has finished. Note that the NJS insert commands to
    # ensure that something is _always_ written to both stdout and stderr.
#    if($out_name eq "") {

# ACHIM:
# this needs to be changed: when one of the files is there, everything is ok
# otherwise report an error
### old:
#    if(! -e $out_name) {
#      failed_report("TSI_STILLEXECUTING. stdout file not found\n");
#      return;
#    }
#    if($err_name eq "") {
#    if(! -e $err_name) {
#      failed_report("TSI_STILLEXECUTING. stderr file not found\n");
#      return;
#    }
### new:
	my $results = "Line not found\n";
	if ((! -e $out_name) && (! -e $err_name)) {
		debug_report("neither stdout nor stderr were found --> ERROR !");
		$results = "UNICORE EXIT STATUS 1 +\n";
	} else {
		debug_report("everything is fine, we have stdout and/or stderr !");
		# SvdB This is not necessary, the NJS assumes this if "line not found" is returned
		# $results = "UNICORE EXIT STATUS 0 +\n";
		# try to extract err code from stderr
		if (open(FILE, "<$err_name")) {
			$/ = "\n"; # go back to standard for a while (main loop restores to usual marker)
			while(<FILE>) {
				$results = $_ if /UNICORE EXIT STATUS/;
				$results = $results.$_ if /UNICORE DECISION/;            # V4 return DECISON too
			}
			close(FILE);
		} 
		else {
			# File open failed, NJS expects it to be there. So
			# another process could have deleted it (end of that Job?)
			failed_report("stderr file not found(2)");
			return;
		}
	}      
	debug_report("result is: ".$results);

#    # Got the files, extract the result code for the NJS
#    # (if any) is the line with UNICORE EXIT STATUS
#    my $results = "Line not found\n";
#    if( ! open(FILE, "<$err_name") ) {
#        # File open failed, NJS expects it to be there. So
#        # another process could have deleted it (end of that Job?)
#        failed_report("stderr file not found(2)");
#        return;
#    }
#    $/ = "\n"; # go back to standard for a while (main loop restores to usual marker)
#    while(<FILE>) {
#        $results = $_ if /UNICORE EXIT STATUS/;
#    }
#    close(FILE);

    # and dispose of the files (move or delete)
    if($disposition eq "DELETE") {
        unlink $out_name;
        unlink $err_name;
    }

#    else {
#        mkpath $outcome_directory, 0, 0700 unless -e $outcome_directory;
#        chmod 0700, $outcome_directory;
#
#		if( rename($out_name, $outcome_directory."stdout") == 0) {
#			failed_report("Failed to move stdout <$out_name> to outcome directory because: $!");
#			return;
#		}
#        if(rename($err_name, $outcome_directory."stderr") == 0) {
#			failed_report("Failed to move stderr <$err_name> to outcome directory because: $!");
#			return;
#		}
#    }

    ok_report($results);
    print main::CMD_SOCK $results;

}
#
#                   Copyright (c) Fujitsu Ltd 2000 - 2004
#
#                Use and distribution is subject a License.
# A copy was supplied with the distribution (see documentation or the jar file).
#
# This product includes software developed by Fujitsu Limited (http://www.fujitsu.com).
